package Alien::Giflib;
use strict;
use vars qw($VERSION $LIB_VERSION $ARCHIVE $URL);

use Config qw'%Config';

$VERSION = '0.01';
$LIB_VERSION = '4.1.4';
$ARCHIVE = "giflib-$LIB_VERSION.tar.gz";
$URL = "http://mesh.dl.sourceforge.net/sourceforge/libungif/$ARCHIVE";

=head1 NAME

Alien::libjpeg - install a local libjpeg just for Perl

=head1 DESCRIPTION

This module allows you to install a local instance of the C<libjpeg>
library via L<CPAN> (or L<CPANPLUS>).

It does not yet try to find an already existing instance but will
always install a new library. If this is not what you want, submit
patches.

=head1 INSTALLATION

This package downloads the full C<libjpeg> sources as found
at L<http://sourceforge.net/projects/libungif/> at installation
time.

If you don't want the download, create a directory C<src> and put the
compressed file as C<src/...> in the build directory.

=head1 METHODS

This package exports the following methods to query
the installation directories:

=head2 C<prefix();>

Returns the prefix (ie, the directory) where the C<include> and C<lib>
directores will live.
On a standard unix system, this might be C</usr/local/>

For this module, it will be C<$INC{'Alien/Libjpeg'}>.

=cut

my $prefix;
sub prefix {
    return $prefix if $prefix;

    my $ext = '.pm';
    $prefix = join '/', split '::', __PACKAGE__;
    $prefix = File::Spec->rel2abs( $INC{$prefix.$ext} );
    $prefix =~ s/$ext$//;

    return $prefix;
}

=head2 lib();

Returns the full path to the C<lib> directory for the C<zlib>
libraries.
On standard unix systems, this might be C</usr/local/lib>

For this module, it will be C<$INC{'Alien/Libjpeg'}/lib>

=head2 include()

Returns the full path to the C<include> directory for the C<zlib>
libraries.
On standard unix systems, this might be C</usr/local/include>

For this module, it will be C<$INC{'Alien/Libjpeg'}/include>

=cut

for my $sub (qw[lib include]) {
    no strict 'refs';
    *$sub = sub {
                my $self = shift;
                return File::Spec->catdir( $self->prefix, $sub );
            };
}

=head2 C<< version() >>

Returns the version of the zlib libraries installed by this version
of C<Alien::Libjpeg>

=cut

sub version { return $LIB_VERSION; }

=head2 C<< lib_files() >>

Returns a list of full paths to the files that can be found in the
C<< Alien::Libjpeg->lib >> directory. This corresponds to all created
libraries.

=head2 C<< include_files() >>

Returns a list of full paths to the files that can be found in the
C<< Alien::Libjpeg->include >> direcotry. This corresponds to all copied
header files.

=cut

my $files = {
    include => [qw|gif_lib.h|],
    lib     => [ 'libgif' . $Config{_a} ],
};

for my $acc (qw[include_files lib_files]) {
    no strict 'refs';
    *$acc = sub {
                my $self    = shift;
                my ($type)  = $acc =~ /(\w+)_files/;

                return map {
                            File::Spec->catfile( $self->$type(), $_)
                        } @{$files->{$type}};
    }
};

=head2 C<< archive() >>

Returns the archive filename. In this case it's C<...>

=head2 C<< lib_version() >>

Returns the version of the linked library.

=head2 C<< url() >>

Returns the url to fetch the source code from.

=cut

for my $name qw(archive lib_version url) {
  no strict 'refs';
  *$name = sub() { return ${uc $name} };
};

=head1 TODO

=over 4

=item *

Probe for system installed version of C<libjpeg>
instead of always downloading and building.

=head1 LICENSE

The Perl code in this distribution can be distributed under
the same terms as Perl itself. The JPEG code is copyright (c)
2001 by the JFIF and has the following copyright note:

=head1 AUTHOR

The author of the Perl code in this package is Max Maischein
L<corion@cpan.org>. Much of the code was cannibalized from
L<Alien::Zlib>.
The authors of the C<libjpeg> libraries
are noted in the C<README> text file of the C<libjpeg>
distribution.

=cut

1;